@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -67,11 +67,7 @@ def is_whitelisted_host(host):
     Args:
         host: The host of the request as string.
     """
-    whitelist = config.get('content', 'host-blocking-whitelist')
-    if whitelist is None:
-        return False
-
-    for pattern in whitelist:
+    for pattern in config.val.content.host_blocking.whitelist:
         if fnmatch.fnmatch(host, pattern.lower()):
             return True
     return False
@@ -114,16 +110,16 @@ class HostBlocker:
 
         data_dir = standarddir.data()
         self._local_hosts_file = os.path.join(data_dir, 'blocked-hosts')
-        self.on_config_changed()
+        self._update_files()
 
         config_dir = standarddir.config()
         self._config_hosts_file = os.path.join(config_dir, 'blocked-hosts')
 
-        objreg.get('config').changed.connect(self.on_config_changed)
+        config.instance.changed.connect(self._update_files)
 
     def is_blocked(self, url):
         """Check if the given URL (as QUrl) is blocked."""
-        if not config.get('content', 'host-blocking-enabled'):
+        if not config.val.content.host_blocking.enabled:
             return False
         host = url.host()
         return ((host in self._blocked_hosts or
@@ -164,9 +160,9 @@ class HostBlocker:
 
         if not found:
             args = objreg.get('args')
-            if (config.get('content', 'host-block-lists') is not None and
+            if (config.val.content.host_blocking.lists and
                     args.basedir is None and
-                    config.get('content', 'host-blocking-enabled')):
+                    config.val.content.host_blocking.enabled):
                 message.info("Run :adblock-update to get adblock lists.")
 
     @cmdutils.register(instance='host-blocker')
@@ -180,18 +176,15 @@ class HostBlocker:
                               self._config_blocked_hosts)
         self._blocked_hosts = set()
         self._done_count = 0
-        urls = config.get('content', 'host-block-lists')
-        download_manager = objreg.get('qtnetwork-download-manager',
-                                      scope='window', window='last-focused')
-        if urls is None:
-            return
-        for url in urls:
+        download_manager = objreg.get('qtnetwork-download-manager')
+        for url in config.val.content.host_blocking.lists:
             if url.scheme() == 'file':
+                filename = url.toLocalFile()
                 try:
-                    fileobj = open(url.path(), 'rb')
+                    fileobj = open(filename, 'rb')
                 except OSError as e:
                     message.error("adblock: Error while reading {}: {}".format(
-                        url.path(), e.strerror))
+                        filename, e.strerror))
                     continue
                 download = FakeDownload(fileobj)
                 self._in_progress.append(download)
@@ -292,11 +285,10 @@ class HostBlocker:
             message.info("adblock: Read {} hosts from {} sources.".format(
                 len(self._blocked_hosts), self._done_count))
 
-    @config.change_filter('content', 'host-block-lists')
-    def on_config_changed(self):
+    @config.change_filter('content.host_blocking.lists')
+    def _update_files(self):
         """Update files when the config changed."""
-        urls = config.get('content', 'host-block-lists')
-        if urls is None:
+        if not config.val.content.host_blocking.lists:
             try:
                 os.remove(self._local_hosts_file)
             except FileNotFoundError:
