@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2015-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -32,7 +32,7 @@ from qutebrowser.misc import objects, sql
 
 
 # increment to indicate that HistoryCompletion must be regenerated
-_USER_VERSION = 1
+_USER_VERSION = 2
 
 
 class CompletionHistory(sql.SqlTable):
@@ -102,7 +102,8 @@ class WebHistory(sql.SqlTable):
         data = {'url': [], 'title': [], 'last_atime': []}
         # select the latest entry for each url
         q = sql.Query('SELECT url, title, max(atime) AS atime FROM History '
-                      'WHERE NOT redirect GROUP BY url ORDER BY atime asc')
+                      'WHERE NOT redirect and url NOT LIKE "qute://back%" '
+                      'GROUP BY url ORDER BY atime asc')
         for entry in q.run():
             data['url'].append(self._format_completion_url(QUrl(entry.url)))
             data['title'].append(entry.title)
@@ -149,8 +150,8 @@ class WebHistory(sql.SqlTable):
         if force:
             self._do_clear()
         else:
-            message.confirm_async(self._do_clear, title="Clear all browsing "
-                                  "history?")
+            message.confirm_async(yes_action=self._do_clear,
+                                  title="Clear all browsing history?")
 
     def _do_clear(self):
         with self._handle_sql_errors():
@@ -171,7 +172,9 @@ class WebHistory(sql.SqlTable):
     @pyqtSlot(QUrl, QUrl, str)
     def add_from_tab(self, url, requested_url, title):
         """Add a new history entry as slot, called from a BrowserTab."""
-        if url.scheme() == 'data' or requested_url.scheme() == 'data':
+        if any(url.scheme() in ('data', 'view-source') or
+               (url.scheme(), url.host()) == ('qute', 'back')
+               for url in (url, requested_url)):
             return
         if url.isEmpty():
             # things set via setHtml
@@ -268,6 +271,7 @@ class WebHistory(sql.SqlTable):
             return
 
         def action():
+            """Actually run the import."""
             with debug.log_time(log.init, 'Import old history file to sqlite'):
                 try:
                     self._read(path)
@@ -340,7 +344,7 @@ class WebHistory(sql.SqlTable):
                 f.write('\n'.join(lines))
             message.info("Dumped history to {}".format(dest))
         except OSError as e:
-            raise cmdexc.CommandError('Could not write history: {}', e)
+            raise cmdexc.CommandError('Could not write history: {}'.format(e))
 
 
 def init(parent=None):
