@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -20,6 +20,7 @@
 """Download manager."""
 
 import io
+import os.path
 import shutil
 import functools
 
@@ -198,13 +199,26 @@ class DownloadItem(downloads.AbstractDownloadItem):
 
     def _ask_confirm_question(self, title, msg):
         no_action = functools.partial(self.cancel, remove_data=False)
+        url = 'file://{}'.format(self._filename)
         message.confirm_async(title=title, text=msg,
                               yes_action=self._after_set_filename,
                               no_action=no_action, cancel_action=no_action,
-                              abort_on=[self.cancelled, self.error])
+                              abort_on=[self.cancelled, self.error], url=url)
+
+    def _ask_create_parent_question(self, title, msg,
+                                    force_overwrite, remember_directory):
+        no_action = functools.partial(self.cancel, remove_data=False)
+        url = 'file://{}'.format(os.path.dirname(self._filename))
+        message.confirm_async(title=title, text=msg,
+                              yes_action=(lambda:
+                                          self._after_create_parent_question(
+                                              force_overwrite,
+                                              remember_directory)),
+                              no_action=no_action, cancel_action=no_action,
+                              abort_on=[self.cancelled, self.error], url=url)
 
     def _set_fileobj(self, fileobj, *, autoclose=True):
-        """"Set the file object to write the download to.
+        """Set the file object to write the download to.
 
         Args:
             fileobj: A file-like object.
@@ -292,8 +306,7 @@ class DownloadItem(downloads.AbstractDownloadItem):
         """Handle QNetworkReply errors."""
         if code == QNetworkReply.OperationCanceledError:
             return
-        else:
-            self._die(self._reply.errorString())
+        self._die(self._reply.errorString())
 
     @pyqtSlot()
     def _on_read_timer_timeout(self):
@@ -368,10 +381,10 @@ class DownloadManager(downloads.AbstractDownloadManager):
         _networkmanager: A NetworkManager for generic downloads.
     """
 
-    def __init__(self, win_id, parent=None):
+    def __init__(self, parent=None):
         super().__init__(parent)
         self._networkmanager = networkmanager.NetworkManager(
-            win_id=win_id, tab_id=None,
+            win_id=None, tab_id=None,
             private=config.val.content.private_browsing, parent=self)
 
     @pyqtSlot('QUrl')
@@ -388,7 +401,7 @@ class DownloadManager(downloads.AbstractDownloadManager):
         """
         if not url.isValid():
             urlutils.invalid_url_error(url, "start download")
-            return
+            return None
         req = QNetworkRequest(url)
         if user_agent is not None:
             req.setHeader(QNetworkRequest.UserAgentHeader, user_agent)
