@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -45,6 +45,10 @@ class DownloadItem(downloads.AbstractDownloadItem):
         qt_item.downloadProgress.connect(self.stats.on_download_progress)
         qt_item.stateChanged.connect(self._on_state_changed)
 
+        # Ensure wrapped qt_item is deleted manually when the wrapper object
+        # is deleted. See https://github.com/qutebrowser/qutebrowser/issues/3373
+        self.destroyed.connect(self._qt_item.deleteLater)
+
     def _is_page_download(self):
         """Check if this item is a page (i.e. mhtml) download."""
         return (self._qt_item.savePageFormat() !=
@@ -96,9 +100,15 @@ class DownloadItem(downloads.AbstractDownloadItem):
         self._qt_item.cancel()
 
     def retry(self):
-        # https://bugreports.qt.io/browse/QTBUG-56840
-        raise downloads.UnsupportedOperationError(
-            "Retrying downloads is unsupported with QtWebEngine")
+        state = self._qt_item.state()
+        assert state == QWebEngineDownloadItem.DownloadInterrupted, state
+
+        try:
+            self._qt_item.resume()
+        except AttributeError:
+            raise downloads.UnsupportedOperationError(
+                "Retrying downloads is unsupported with QtWebEngine on "
+                "Qt/PyQt < 5.10")
 
     def _get_open_filename(self):
         return self._filename
@@ -125,6 +135,7 @@ class DownloadItem(downloads.AbstractDownloadItem):
         question = usertypes.Question()
         question.title = title
         question.text = msg
+        question.url = 'file://{}'.format(self._filename)
         question.mode = usertypes.PromptMode.yesno
         question.answered_yes.connect(self._after_set_filename)
         question.answered_no.connect(no_action)
@@ -133,6 +144,23 @@ class DownloadItem(downloads.AbstractDownloadItem):
         self.error.connect(question.abort)
         message.global_bridge.ask(question, blocking=True)
 
+    def _ask_create_parent_question(self, title, msg,
+                                    force_overwrite, remember_directory):
+        no_action = functools.partial(self.cancel, remove_data=False)
+        question = usertypes.Question()
+        question.title = title
+        question.text = msg
+        question.url = 'file://{}'.format(os.path.dirname(self._filename))
+        question.mode = usertypes.PromptMode.yesno
+        question.answered_yes.connect(lambda:
+                                      self._after_create_parent_question(
+                                          force_overwrite, remember_directory))
+        question.answered_no.connect(no_action)
+        question.cancelled.connect(no_action)
+        self.cancelled.connect(question.abort)
+        self.error.connect(question.abort)
+        message.global_bridge.ask(question, blocking=True)
+
     def _after_set_filename(self):
         self._qt_item.setPath(self._filename)
         self._qt_item.accept()
