@@ -1,7 +1,7 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
-# Copyright 2015-2017 Antoni Boucher (antoyo) <bouanto@zoho.com>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2015-2018 Antoni Boucher (antoyo) <bouanto@zoho.com>
 #
 # This file is part of qutebrowser.
 #
@@ -25,7 +25,7 @@
 
 import os
 
-from qutebrowser.browser.webkit.network import schemehandler, networkreply
+from qutebrowser.browser.webkit.network import networkreply
 from qutebrowser.utils import jinja
 
 
@@ -111,23 +111,21 @@ def dirbrowser_html(path):
     return html.encode('UTF-8', errors='xmlcharrefreplace')
 
 
-class FileSchemeHandler(schemehandler.SchemeHandler):
+def handler(request):
+    """Handler for a file:// URL.
 
-    """Scheme handler for file: URLs."""
+    Args:
+        request: QNetworkRequest to answer to.
 
-    def createRequest(self, _op, request, _outgoing_data):
-        """Create a new request.
-
-        Args:
-             request: const QNetworkRequest & req
-             _op: Operation op
-             _outgoing_data: QIODevice * outgoingData
-
-        Return:
-            A QNetworkReply for directories, None for files.
-        """
-        path = request.url().toLocalFile()
+    Return:
+        A QNetworkReply for directories, None for files.
+    """
+    path = request.url().toLocalFile()
+    try:
         if os.path.isdir(path):
             data = dirbrowser_html(path)
             return networkreply.FixedDataNetworkReply(
-                request, data, 'text/html', self.parent())
+                request, data, 'text/html')
+        return None
+    except UnicodeEncodeError:
+        return None
