@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -35,6 +35,7 @@ class CompletionInfo:
 
     config = attr.ib()
     keyconf = attr.ib()
+    win_id = attr.ib()
 
 
 class Completer(QObject):
@@ -43,7 +44,7 @@ class Completer(QObject):
 
     Attributes:
         _cmd: The statusbar Command object this completer belongs to.
-        _ignore_change: Whether to ignore the next completion update.
+        _win_id: The id of the window that owns this object.
         _timer: The timer used to trigger the completion update.
         _last_cursor_pos: The old cursor position so we avoid double completion
                           updates.
@@ -51,15 +52,15 @@ class Completer(QObject):
         _last_completion_func: The completion function used for the last text.
     """
 
-    def __init__(self, cmd, parent=None):
+    def __init__(self, *, cmd, win_id, parent=None):
         super().__init__(parent)
         self._cmd = cmd
-        self._ignore_change = False
+        self._win_id = win_id
         self._timer = QTimer()
         self._timer.setSingleShot(True)
         self._timer.setInterval(0)
         self._timer.timeout.connect(self._update_completion)
-        self._last_cursor_pos = None
+        self._last_cursor_pos = -1
         self._last_text = None
         self._last_completion_func = None
         self._cmd.update_completion.connect(self.schedule_completion_update)
@@ -86,8 +87,6 @@ class Completer(QObject):
             # cursor on a flag or after an explicit split (--)
             return None
         log.completion.debug("Before removing flags: {}".format(before_cursor))
-        before_cursor = [x for x in before_cursor if not x.startswith('-')]
-        log.completion.debug("After removing flags: {}".format(before_cursor))
         if not before_cursor:
             # '|' or 'set|'
             log.completion.debug('Starting command completion')
@@ -98,6 +97,9 @@ class Completer(QObject):
             log.completion.debug("No completion for unknown command: {}"
                                  .format(before_cursor[0]))
             return None
+
+        before_cursor = [x for x in before_cursor if not x.startswith('-')]
+        log.completion.debug("After removing flags: {}".format(before_cursor))
         argpos = len(before_cursor) - 1
         try:
             func = cmd.get_pos_arg_info(argpos).completion
@@ -133,9 +135,7 @@ class Completer(QObject):
             return [], '', []
         parser = runners.CommandParser()
         result = parser.parse(text, fallback=True, keep=True)
-        # pylint: disable=not-an-iterable
         parts = [x for x in result.cmdline if x]
-        # pylint: enable=not-an-iterable
         pos = self._cmd.cursorPosition() - len(self._cmd.prefix())
         pos = min(pos, len(text))  # Qt treats 2-byte UTF-16 chars as 2 chars
         log.completion.debug('partitioning {} around position {}'.format(parts,
@@ -154,8 +154,7 @@ class Completer(QObject):
                     "partitioned: {} '{}' {}".format(prefix, center, postfix))
                 return prefix, center, postfix
 
-        # We should always return above
-        assert False, parts
+        raise utils.Unreachable("Not all parts consumed: {}".format(parts))
 
     @pyqtSlot(str)
     def on_selection_changed(self, text):
@@ -178,16 +177,16 @@ class Completer(QObject):
             text = self._quote(text)
         model = self._model()
         if model.count() == 1 and config.val.completion.quick:
-            # If we only have one item, we want to apply it immediately
-            # and go on to the next part.
-            self._change_completed_part(text, before, after, immediate=True)
+            # If we only have one item, we want to apply it immediately and go
+            # on to the next part, unless we are quick-completing the part
+            # after maxsplit, so that we don't keep offering completions
+            # (see issue #1519)
             if maxsplit is not None and maxsplit < len(before):
-                # If we are quick-completing the part after maxsplit, don't
-                # keep offering completions (see issue #1519)
-                self._ignore_change = True
+                self._change_completed_part(text, before, after)
+            else:
+                self._change_completed_part(text, before, after,
+                                            immediate=True)
         else:
-            log.completion.debug("Will ignore next completion update.")
-            self._ignore_change = True
             self._change_completed_part(text, before, after)
 
     @pyqtSlot()
@@ -196,26 +195,31 @@ class Completer(QObject):
 
         For performance reasons we don't want to block here, instead we do this
         in the background.
+
+        We delay the update only if we've already input some text and ignore
+        updates if the text is shorter than completion.min_chars (unless we're
+        hitting backspace in which case updates won't be ignored).
         """
-        if (self._cmd.cursorPosition() == self._last_cursor_pos and
-                self._cmd.text() == self._last_text):
+        _cmd, _sep, rest = self._cmd.text().partition(' ')
+        input_length = len(rest)
+        if (0 < input_length < config.val.completion.min_chars and
+                self._cmd.cursorPosition() > self._last_cursor_pos):
+            log.completion.debug("Ignoring update because the length of "
+                                 "the text is less than completion.min_chars.")
+        elif (self._cmd.cursorPosition() == self._last_cursor_pos and
+              self._cmd.text() == self._last_text):
             log.completion.debug("Ignoring update because there were no "
                                  "changes.")
         else:
             log.completion.debug("Scheduling completion update.")
-            self._timer.start()
+            start_delay = config.val.completion.delay if self._last_text else 0
+            self._timer.start(start_delay)
         self._last_cursor_pos = self._cmd.cursorPosition()
         self._last_text = self._cmd.text()
 
     @pyqtSlot()
     def _update_completion(self):
         """Check if completions are available and activate them."""
-        if self._ignore_change:
-            log.completion.debug("Ignoring completion update because "
-                                 "ignore_change is True.")
-            self._ignore_change = False
-            return
-
         completion = self.parent()
 
         if self._cmd.prefix() != ':':
@@ -244,10 +248,11 @@ class Completer(QObject):
         if func != self._last_completion_func:
             self._last_completion_func = func
             args = (x for x in before_cursor[1:] if not x.startswith('-'))
-            with debug.log_time(log.completion,
-                    'Starting {} completion'.format(func.__name__)):
+            with debug.log_time(log.completion, 'Starting {} completion'
+                                .format(func.__name__)):
                 info = CompletionInfo(config=config.instance,
-                                      keyconf=config.key_instance)
+                                      keyconf=config.key_instance,
+                                      win_id=self._win_id)
                 model = func(*args, info=info)
             with debug.log_time(log.completion, 'Set completion model'):
                 completion.set_model(model)
@@ -273,7 +278,20 @@ class Completer(QObject):
             # pad with a space if quick-completing the last entry
             text += ' '
         log.completion.debug("setting text = '{}', pos = {}".format(text, pos))
+
+        # generally, we don't want to let self._cmd emit cursorPositionChanged,
+        # because that'll schedule a completion update. That happens when
+        # tabbing through the completions, and we want to change the command
+        # text but we also want to keep the original completion list for the
+        # command the user manually entered. The exception is when we're
+        # immediately completing, in which case we *do* want to update the
+        # completion view so that we can start completing the next part
+        if not immediate:
+            self._cmd.blockSignals(True)
+
         self._cmd.setText(text)
         self._cmd.setCursorPosition(pos)
         self._cmd.setFocus()
+
+        self._cmd.blockSignals(False)
         self._cmd.show_cmd.emit()
