@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,13 +22,15 @@
 from qutebrowser.config import configdata, configexc
 from qutebrowser.completion.models import completionmodel, listcategory, util
 from qutebrowser.commands import runners, cmdexc
+from qutebrowser.keyinput import keyutils
 
 
 def option(*, info):
     """A CompletionModel filled with settings and their descriptions."""
     model = completionmodel.CompletionModel(column_widths=(20, 70, 10))
     options = ((opt.name, opt.description, info.config.get_str(opt.name))
-               for opt in configdata.DATA.values())
+               for opt in configdata.DATA.values()
+               if not opt.no_autoconfig)
     model.add_category(listcategory.ListCategory("Options", options))
     return model
 
@@ -36,8 +38,10 @@ def option(*, info):
 def customized_option(*, info):
     """A CompletionModel filled with set settings and their descriptions."""
     model = completionmodel.CompletionModel(column_widths=(20, 70, 10))
-    options = ((opt.name, opt.description, info.config.get_str(opt.name))
-               for opt, _value in info.config)
+    options = ((values.opt.name, values.opt.description,
+                info.config.get_str(values.opt.name))
+               for values in info.config
+               if values)
     model.add_category(listcategory.ListCategory("Customized options",
                                                  options))
     return model
@@ -60,7 +64,8 @@ def value(optname, *_values, info):
 
     opt = info.config.get_opt(optname)
     default = opt.typ.to_str(opt.default)
-    cur_cat = listcategory.ListCategory("Current/Default",
+    cur_cat = listcategory.ListCategory(
+        "Current/Default",
         [(current, "Current value"), (default, "Default value")])
     model.add_category(cur_cat)
 
@@ -70,6 +75,34 @@ def value(optname, *_values, info):
     return model
 
 
+def _bind_current_default(key, info):
+    """Get current/default data for the given key."""
+    data = []
+    try:
+        seq = keyutils.KeySequence.parse(key)
+    except keyutils.KeyParseError as e:
+        data.append(('', str(e), key))
+        return data
+
+    cmd_text = info.keyconf.get_command(seq, 'normal')
+    if cmd_text:
+        parser = runners.CommandParser()
+        try:
+            cmd = parser.parse(cmd_text).cmd
+        except cmdexc.NoSuchCommandError:
+            data.append((cmd_text, '(Current) Invalid command!', key))
+        else:
+            data.append((cmd_text, '(Current) {}'.format(cmd.desc), key))
+
+    cmd_text = info.keyconf.get_command(seq, 'normal', default=True)
+    if cmd_text:
+        parser = runners.CommandParser()
+        cmd = parser.parse(cmd_text).cmd
+        data.append((cmd_text, '(Default) {}'.format(cmd.desc), key))
+
+    return data
+
+
 def bind(key, *, info):
     """A CompletionModel filled with all bindable commands and descriptions.
 
@@ -77,17 +110,10 @@ def bind(key, *, info):
         key: the key being bound.
     """
     model = completionmodel.CompletionModel(column_widths=(20, 60, 20))
-    cmd_text = info.keyconf.get_command(key, 'normal')
+    data = _bind_current_default(key, info)
 
-    if cmd_text:
-        parser = runners.CommandParser()
-        try:
-            cmd = parser.parse(cmd_text).cmd
-        except cmdexc.NoSuchCommandError:
-            data = [(cmd_text, 'Invalid command!', key)]
-        else:
-            data = [(cmd_text, cmd.desc, key)]
-        model.add_category(listcategory.ListCategory("Current", data))
+    if data:
+        model.add_category(listcategory.ListCategory("Current/Default", data))
 
     cmdlist = util.get_cmd_completions(info, include_hidden=True,
                                        include_aliases=True)
