@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -25,9 +25,10 @@ import functools
 
 from PyQt5.QtCore import pyqtSignal, pyqtSlot, QObject
 
-from qutebrowser.config import configdata, configexc
+from qutebrowser.config import configdata, configexc, configutils
 from qutebrowser.utils import utils, log, jinja
 from qutebrowser.misc import objects
+from qutebrowser.keyinput import keyutils
 
 # An easy way to access the config from other code via config.val.foo
 val = None
@@ -37,8 +38,11 @@ key_instance = None
 # Keeping track of all change filters to validate them later.
 change_filters = []
 
+# Sentinel
+UNSET = object()
 
-class change_filter:  # pylint: disable=invalid-name
+
+class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
 
     """Decorator to filter calls based on a config section/option matching.
 
@@ -104,13 +108,17 @@ class change_filter:  # pylint: disable=invalid-name
         if self._function:
             @functools.wraps(func)
             def wrapper(option=None):
+                """Call the underlying function."""
                 if self._check_match(option):
                     return func()
+                return None
         else:
             @functools.wraps(func)
             def wrapper(wrapper_self, option=None):
+                """Call the underlying function."""
                 if self._check_match(option):
                     return func(wrapper_self)
+                return None
 
         return wrapper
 
@@ -128,20 +136,18 @@ class KeyConfig:
     def __init__(self, config):
         self._config = config
 
-    def _prepare(self, key, mode):
-        """Make sure the given mode exists and normalize the key."""
+    def _validate(self, key, mode):
+        """Validate the given key and mode."""
+        # Catch old usage of this code
+        assert isinstance(key, keyutils.KeySequence), key
         if mode not in configdata.DATA['bindings.default'].default:
             raise configexc.KeybindingError("Invalid mode {}!".format(mode))
-        if utils.is_special_key(key):
-            # <Ctrl-t>, <ctrl-T>, and <ctrl-t> should be considered equivalent
-            return utils.normalize_keystr(key)
-        return key
 
     def get_bindings_for(self, mode):
         """Get the combined bindings for the given mode."""
         bindings = dict(val.bindings.default[mode])
         for key, binding in val.bindings.commands[mode].items():
-            if binding is None:
+            if not binding:
                 bindings.pop(key, None)
             else:
                 bindings[key] = binding
@@ -151,21 +157,24 @@ class KeyConfig:
         """Get a dict of commands to a list of bindings for the mode."""
         cmd_to_keys = {}
         bindings = self.get_bindings_for(mode)
-        for key, full_cmd in sorted(bindings.items()):
+        for seq, full_cmd in sorted(bindings.items()):
             for cmd in full_cmd.split(';;'):
                 cmd = cmd.strip()
                 cmd_to_keys.setdefault(cmd, [])
-                # put special bindings last
-                if utils.is_special_key(key):
-                    cmd_to_keys[cmd].append(key)
+                # Put bindings involving modifiers last
+                if any(info.modifiers for info in seq):
+                    cmd_to_keys[cmd].append(str(seq))
                 else:
-                    cmd_to_keys[cmd].insert(0, key)
+                    cmd_to_keys[cmd].insert(0, str(seq))
         return cmd_to_keys
 
-    def get_command(self, key, mode):
+    def get_command(self, key, mode, default=False):
         """Get the command for a given key (or None)."""
-        key = self._prepare(key, mode)
-        bindings = self.get_bindings_for(mode)
+        self._validate(key, mode)
+        if default:
+            bindings = dict(val.bindings.default[mode])
+        else:
+            bindings = self.get_bindings_for(mode)
         return bindings.get(key, None)
 
     def bind(self, key, command, *, mode, save_yaml=False):
@@ -175,23 +184,23 @@ class KeyConfig:
                 "Can't add binding '{}' with empty command in {} "
                 'mode'.format(key, mode))
 
-        key = self._prepare(key, mode)
+        self._validate(key, mode)
         log.keyboard.vdebug("Adding binding {} -> {} in mode {}.".format(
             key, command, mode))
 
-        bindings = self._config.get_obj('bindings.commands')
+        bindings = self._config.get_mutable_obj('bindings.commands')
         if mode not in bindings:
             bindings[mode] = {}
-        bindings[mode][key] = command
+        bindings[mode][str(key)] = command
         self._config.update_mutables(save_yaml=save_yaml)
 
     def bind_default(self, key, *, mode='normal', save_yaml=False):
         """Restore a default keybinding."""
-        key = self._prepare(key, mode)
+        self._validate(key, mode)
 
-        bindings_commands = self._config.get_obj('bindings.commands')
+        bindings_commands = self._config.get_mutable_obj('bindings.commands')
         try:
-            del bindings_commands[mode][key]
+            del bindings_commands[mode][str(key)]
         except KeyError:
             raise configexc.KeybindingError(
                 "Can't find binding '{}' in {} mode".format(key, mode))
@@ -199,18 +208,18 @@ class KeyConfig:
 
     def unbind(self, key, *, mode='normal', save_yaml=False):
         """Unbind the given key in the given mode."""
-        key = self._prepare(key, mode)
+        self._validate(key, mode)
 
-        bindings_commands = self._config.get_obj('bindings.commands')
+        bindings_commands = self._config.get_mutable_obj('bindings.commands')
 
         if val.bindings.commands[mode].get(key, None) is not None:
             # In custom bindings -> remove it
-            del bindings_commands[mode][key]
+            del bindings_commands[mode][str(key)]
         elif key in val.bindings.default[mode]:
             # In default bindings -> shadow it with None
             if mode not in bindings_commands:
                 bindings_commands[mode] = {}
-            bindings_commands[mode][key] = None
+            bindings_commands[mode][str(key)] = None
         else:
             raise configexc.KeybindingError(
                 "Can't find binding '{}' in {} mode".format(key, mode))
@@ -222,8 +231,12 @@ class Config(QObject):
 
     """Main config object.
 
+    Class attributes:
+        MUTABLE_TYPES: Types returned from the config which could potentially
+        be mutated.
+
     Attributes:
-        _values: A dict mapping setting names to their values.
+        _values: A dict mapping setting names to configutils.Values objects.
         _mutables: A dictionary of mutable objects to be checked for changes.
         _yaml: A YamlConfig object or None.
 
@@ -231,19 +244,25 @@ class Config(QObject):
         changed: Emitted with the option name when an option changed.
     """
 
+    MUTABLE_TYPES = (dict, list)
     changed = pyqtSignal(str)
 
     def __init__(self, yaml_config, parent=None):
         super().__init__(parent)
         self.changed.connect(_render_stylesheet.cache_clear)
-        self._values = {}
         self._mutables = {}
         self._yaml = yaml_config
+        self._init_values()
+
+    def _init_values(self):
+        """Populate the self._values dict."""
+        self._values = {}
+        for name, opt in configdata.DATA.items():
+            self._values[name] = configutils.Values(opt)
 
     def __iter__(self):
-        """Iterate over Option, value tuples."""
-        for name, value in sorted(self._values.items()):
-            yield (self.get_opt(name), value)
+        """Iterate over configutils.Values items."""
+        yield from self._values.values()
 
     def init_save_manager(self, save_manager):
         """Make sure the config gets saved properly.
@@ -253,24 +272,32 @@ class Config(QObject):
         """
         self._yaml.init_save_manager(save_manager)
 
-    def _set_value(self, opt, value):
+    def _set_value(self, opt, value, pattern=None):
         """Set the given option to the given value."""
         if not isinstance(objects.backend, objects.NoBackend):
             if objects.backend not in opt.backends:
-                raise configexc.BackendError(objects.backend)
+                raise configexc.BackendError(opt.name, objects.backend)
 
         opt.typ.to_py(value)  # for validation
-        self._values[opt.name] = opt.typ.from_obj(value)
+
+        self._values[opt.name].add(opt.typ.from_obj(value), pattern)
 
         self.changed.emit(opt.name)
         log.config.debug("Config option changed: {} = {}".format(
             opt.name, value))
 
+    def _check_yaml(self, opt, save_yaml):
+        """Make sure the given option may be set in autoconfig.yml."""
+        if save_yaml and opt.no_autoconfig:
+            raise configexc.NoAutoconfigError(opt.name)
+
     def read_yaml(self):
         """Read the YAML settings from self._yaml."""
         self._yaml.load()
-        for name, value in self._yaml:
-            self._set_value(self.get_opt(name), value)
+        for values in self._yaml:
+            for scoped in values:
+                self._set_value(values.opt, scoped.value,
+                                pattern=scoped.pattern)
 
     def get_opt(self, name):
         """Get a configdata.Option object for the given setting."""
@@ -283,77 +310,115 @@ class Config(QObject):
                 name, deleted=deleted, renamed=renamed)
             raise exception from None
 
-    def get(self, name):
+    def get(self, name, url=None):
         """Get the given setting converted for Python code."""
         opt = self.get_opt(name)
-        obj = self.get_obj(name, mutable=False)
+        obj = self.get_obj(name, url=url)
         return opt.typ.to_py(obj)
 
-    def get_obj(self, name, *, mutable=True):
+    def _maybe_copy(self, value):
+        """Copy the value if it could potentially be mutated."""
+        if isinstance(value, self.MUTABLE_TYPES):
+            # For mutable objects, create a copy so we don't accidentally
+            # mutate the config's internal value.
+            return copy.deepcopy(value)
+        else:
+            # Shouldn't be mutable (and thus hashable)
+            assert value.__hash__ is not None, value
+            return value
+
+    def get_obj(self, name, *, url=None):
         """Get the given setting as object (for YAML/config.py).
 
-        If mutable=True is set, watch the returned object for mutations.
+        Note that the returned values are not watched for mutation.
+        If a URL is given, return the value which should be used for that URL.
         """
-        opt = self.get_opt(name)
-        obj = None
+        self.get_opt(name)  # To make sure it exists
+        value = self._values[name].get_for_url(url)
+        return self._maybe_copy(value)
+
+    def get_obj_for_pattern(self, name, *, pattern):
+        """Get the given setting as object (for YAML/config.py).
+
+        This gets the overridden value for a given pattern, or
+        configutils.UNSET if no such override exists.
+        """
+        self.get_opt(name)  # To make sure it exists
+        value = self._values[name].get_for_pattern(pattern, fallback=False)
+        return self._maybe_copy(value)
+
+    def get_mutable_obj(self, name, *, pattern=None):
+        """Get an object which can be mutated, e.g. in a config.py.
+
+        If a pattern is given, return the value for that pattern.
+        Note that it's impossible to get a mutable object for an URL as we
+        wouldn't know what pattern to apply.
+        """
+        self.get_opt(name)  # To make sure it exists
+
         # If we allow mutation, there is a chance that prior mutations already
         # entered the mutable dictionary and thus further copies are unneeded
         # until update_mutables() is called
-        if name in self._mutables and mutable:
+        if name in self._mutables:
             _copy, obj = self._mutables[name]
-        # Otherwise, we return a copy of the value stored internally, so the
-        # internal value can never be changed by mutating the object returned.
-        else:
-            obj = copy.deepcopy(self._values.get(name, opt.default))
-            # Then we watch the returned object for changes.
-            if isinstance(obj, (dict, list)):
-                if mutable:
-                    self._mutables[name] = (copy.deepcopy(obj), obj)
-            else:
-                # Shouldn't be mutable (and thus hashable)
-                assert obj.__hash__ is not None, obj
-        return obj
+            return obj
 
-    def get_str(self, name):
-        """Get the given setting as string."""
+        value = self._values[name].get_for_pattern(pattern)
+        copy_value = self._maybe_copy(value)
+
+        # Watch the returned object for changes if it's mutable.
+        if isinstance(copy_value, self.MUTABLE_TYPES):
+            self._mutables[name] = (value, copy_value)  # old, new
+
+        return copy_value
+
+    def get_str(self, name, *, pattern=None):
+        """Get the given setting as string.
+
+        If a pattern is given, get the setting for the given pattern or
+        configutils.UNSET.
+        """
         opt = self.get_opt(name)
-        value = self._values.get(name, opt.default)
+        values = self._values[name]
+        value = values.get_for_pattern(pattern)
         return opt.typ.to_str(value)
 
-    def set_obj(self, name, value, *, save_yaml=False):
+    def set_obj(self, name, value, *, pattern=None, save_yaml=False):
         """Set the given setting from a YAML/config.py object.
 
         If save_yaml=True is given, store the new value to YAML.
         """
-        self._set_value(self.get_opt(name), value)
+        opt = self.get_opt(name)
+        self._check_yaml(opt, save_yaml)
+        self._set_value(opt, value, pattern=pattern)
         if save_yaml:
-            self._yaml[name] = value
+            self._yaml.set_obj(name, value, pattern=pattern)
 
-    def set_str(self, name, value, *, save_yaml=False):
+    def set_str(self, name, value, *, pattern=None, save_yaml=False):
         """Set the given setting from a string.
 
         If save_yaml=True is given, store the new value to YAML.
         """
         opt = self.get_opt(name)
+        self._check_yaml(opt, save_yaml)
         converted = opt.typ.from_str(value)
         log.config.debug("Setting {} (type {}) to {!r} (converted from {!r})"
                          .format(name, opt.typ.__class__.__name__, converted,
                                  value))
-        self._set_value(opt, converted)
+        self._set_value(opt, converted, pattern=pattern)
         if save_yaml:
-            self._yaml[name] = converted
+            self._yaml.set_obj(name, converted, pattern=pattern)
 
-    def unset(self, name, *, save_yaml=False):
+    def unset(self, name, *, save_yaml=False, pattern=None):
         """Set the given setting back to its default."""
-        self.get_opt(name)
-        try:
-            del self._values[name]
-        except KeyError:
-            return
-        self.changed.emit(name)
+        opt = self.get_opt(name)
+        self._check_yaml(opt, save_yaml)
+        changed = self._values[name].remove(pattern)
+        if changed:
+            self.changed.emit(name)
 
         if save_yaml:
-            self._yaml.unset(name)
+            self._yaml.unset(name, pattern=pattern)
 
     def clear(self, *, save_yaml=False):
         """Clear all settings in the config.
@@ -361,10 +426,10 @@ class Config(QObject):
         If save_yaml=True is given, also remove all customization from the YAML
         file.
         """
-        old_values = self._values
-        self._values = {}
-        for name in old_values:
-            self.changed.emit(name)
+        for name, values in self._values.items():
+            if values:
+                values.clear()
+                self.changed.emit(name)
 
         if save_yaml:
             self._yaml.clear()
@@ -390,13 +455,15 @@ class Config(QObject):
         Return:
             The changed config part as string.
         """
-        lines = []
-        for opt, value in self:
-            str_value = opt.typ.to_str(value)
-            lines.append('{} = {}'.format(opt.name, str_value))
-        if not lines:
-            lines = ['<Default configuration>']
-        return '\n'.join(lines)
+        blocks = []
+        for values in sorted(self, key=lambda v: v.opt.name):
+            if values:
+                blocks.append(str(values))
+
+        if not blocks:
+            return '<Default configuration>'
+
+        return '\n'.join(blocks)
 
 
 class ConfigContainer:
@@ -408,16 +475,21 @@ class ConfigContainer:
         _prefix: The __getattr__ chain leading up to this object.
         _configapi: If given, get values suitable for config.py and
                     add errors to the given ConfigAPI object.
+        _pattern: The URL pattern to be used.
     """
 
-    def __init__(self, config, configapi=None, prefix=''):
+    def __init__(self, config, configapi=None, prefix='', pattern=None):
         self._config = config
         self._prefix = prefix
         self._configapi = configapi
+        self._pattern = pattern
+        if configapi is None and pattern is not None:
+            raise TypeError("Can't use pattern without configapi!")
 
     def __repr__(self):
         return utils.get_repr(self, constructor=True, config=self._config,
-                              configapi=self._configapi, prefix=self._prefix)
+                              configapi=self._configapi, prefix=self._prefix,
+                              pattern=self._pattern)
 
     @contextlib.contextmanager
     def _handle_error(self, action, name):
@@ -445,7 +517,7 @@ class ConfigContainer:
         if configdata.is_valid_prefix(name):
             return ConfigContainer(config=self._config,
                                    configapi=self._configapi,
-                                   prefix=name)
+                                   prefix=name, pattern=self._pattern)
 
         with self._handle_error('getting', name):
             if self._configapi is None:
@@ -453,16 +525,18 @@ class ConfigContainer:
                 return self._config.get(name)
             else:
                 # access from config.py
-                return self._config.get_obj(name)
+                return self._config.get_mutable_obj(
+                    name, pattern=self._pattern)
 
     def __setattr__(self, attr, value):
         """Set the given option in the config."""
         if attr.startswith('_'):
-            return super().__setattr__(attr, value)
+            super().__setattr__(attr, value)
+            return
 
         name = self._join(attr)
         with self._handle_error('setting', name):
-            self._config.set_obj(name, value)
+            self._config.set_obj(name, value, pattern=self._pattern)
 
     def _join(self, attr):
         """Get the prefix joined with the given attribute."""
@@ -483,8 +557,8 @@ def set_register_stylesheet(obj, *, stylesheet=None, update=True):
         stylesheet: The stylesheet to use.
         update: Whether to update the stylesheet on config changes.
     """
-    observer = StyleSheetObserver(obj, stylesheet=stylesheet)
-    observer.register(update=update)
+    observer = StyleSheetObserver(obj, stylesheet, update)
+    observer.register()
 
 
 @functools.lru_cache()
@@ -504,9 +578,14 @@ class StyleSheetObserver(QObject):
         _stylesheet: The stylesheet template to use.
     """
 
-    def __init__(self, obj, stylesheet):
-        super().__init__(parent=obj)
+    def __init__(self, obj, stylesheet, update):
+        super().__init__()
         self._obj = obj
+        self._update = update
+
+        # We only need to hang around if we are asked to update.
+        if self._update:
+            self.setParent(self._obj)
         if stylesheet is None:
             self._stylesheet = obj.STYLESHEET
         else:
@@ -525,7 +604,7 @@ class StyleSheetObserver(QObject):
         """Update the stylesheet for obj."""
         self._obj.setStyleSheet(self._get_stylesheet())
 
-    def register(self, update):
+    def register(self):
         """Do a first update and listen for more.
 
         Args:
@@ -535,5 +614,5 @@ class StyleSheetObserver(QObject):
         log.config.vdebug("stylesheet for {}: {}".format(
             self._obj.__class__.__name__, qss))
         self._obj.setStyleSheet(qss)
-        if update:
+        if self._update:
             instance.changed.connect(self._update_stylesheet)
