@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -32,8 +32,9 @@ import yaml
 from PyQt5.QtCore import pyqtSignal, QObject, QSettings
 
 import qutebrowser
-from qutebrowser.config import configexc, config, configdata
-from qutebrowser.utils import standarddir, utils, qtutils, log
+from qutebrowser.config import configexc, config, configdata, configutils
+from qutebrowser.keyinput import keyutils
+from qutebrowser.utils import standarddir, utils, qtutils, log, urlmatch
 
 
 # The StateConfig instance
@@ -80,16 +81,19 @@ class YamlConfig(QObject):
         VERSION: The current version number of the config file.
     """
 
-    VERSION = 1
+    VERSION = 2
     changed = pyqtSignal()
 
     def __init__(self, parent=None):
         super().__init__(parent)
         self._filename = os.path.join(standarddir.config(auto=True),
                                       'autoconfig.yml')
-        self._values = {}
         self._dirty = None
 
+        self._values = {}
+        for name, opt in configdata.DATA.items():
+            self._values[name] = configutils.Values(opt)
+
     def init_save_manager(self, save_manager):
         """Make sure the config gets saved properly.
 
@@ -98,18 +102,9 @@ class YamlConfig(QObject):
         """
         save_manager.add_saveable('yaml-config', self._save, self.changed)
 
-    def __getitem__(self, name):
-        return self._values[name]
-
-    def __setitem__(self, name, value):
-        self._values[name] = value
-        self._mark_changed()
-
-    def __contains__(self, name):
-        return name in self._values
-
     def __iter__(self):
-        return iter(sorted(self._values.items()))
+        """Iterate over configutils.Values items."""
+        yield from self._values.values()
 
     def _mark_changed(self):
         """Mark the YAML config as changed."""
@@ -121,7 +116,17 @@ class YamlConfig(QObject):
         if not self._dirty:
             return
 
-        data = {'config_version': self.VERSION, 'global': self._values}
+        settings = {}
+        for name, values in sorted(self._values.items()):
+            if not values:
+                continue
+            settings[name] = {}
+            for scoped in values:
+                key = ('global' if scoped.pattern is None
+                       else str(scoped.pattern))
+                settings[name][key] = scoped.value
+
+        data = {'config_version': self.VERSION, 'settings': settings}
         with qtutils.savefile_open(self._filename) as f:
             f.write(textwrap.dedent("""
                 # DO NOT edit this file by hand, qutebrowser will overwrite it.
@@ -130,13 +135,36 @@ class YamlConfig(QObject):
             """.lstrip('\n')))
             utils.yaml_dump(data, f)
 
+    def _pop_object(self, yaml_data, key, typ):
+        """Get a global object from the given data."""
+        if not isinstance(yaml_data, dict):
+            desc = configexc.ConfigErrorDesc("While loading data",
+                                             "Toplevel object is not a dict")
+            raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+
+        if key not in yaml_data:
+            desc = configexc.ConfigErrorDesc(
+                "While loading data",
+                "Toplevel object does not contain '{}' key".format(key))
+            raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+
+        data = yaml_data.pop(key)
+
+        if not isinstance(data, typ):
+            desc = configexc.ConfigErrorDesc(
+                "While loading data",
+                "'{}' object is not a {}".format(key, typ.__name__))
+            raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+
+        return data
+
     def load(self):
         """Load configuration from the configured YAML file."""
         try:
             with open(self._filename, 'r', encoding='utf-8') as f:
                 yaml_data = utils.yaml_load(f)
         except FileNotFoundError:
-            return {}
+            return
         except OSError as e:
             desc = configexc.ConfigErrorDesc("While reading", e)
             raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
@@ -144,59 +172,132 @@ class YamlConfig(QObject):
             desc = configexc.ConfigErrorDesc("While parsing", e)
             raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
 
-        try:
-            global_obj = yaml_data['global']
-        except KeyError:
+        config_version = self._pop_object(yaml_data, 'config_version', int)
+        if config_version == 1:
+            settings = self._load_legacy_settings_object(yaml_data)
+            self._mark_changed()
+        elif config_version > self.VERSION:
             desc = configexc.ConfigErrorDesc(
-                "While loading data",
-                "Toplevel object does not contain 'global' key")
-            raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
-        except TypeError:
-            desc = configexc.ConfigErrorDesc("While loading data",
-                                             "Toplevel object is not a dict")
+                "While reading",
+                "Can't read config from incompatible newer version")
             raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+        else:
+            settings = self._load_settings_object(yaml_data)
+            self._dirty = False
 
-        if not isinstance(global_obj, dict):
-            desc = configexc.ConfigErrorDesc(
-                "While loading data",
-                "'global' object is not a dict")
-            raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+        settings = self._handle_migrations(settings)
+        self._validate(settings)
+        self._build_values(settings)
 
-        self._values = global_obj
-        self._dirty = False
+    def _load_settings_object(self, yaml_data):
+        """Load the settings from the settings: key."""
+        return self._pop_object(yaml_data, 'settings', dict)
 
-        self._handle_migrations()
+    def _load_legacy_settings_object(self, yaml_data):
+        data = self._pop_object(yaml_data, 'global', dict)
+        settings = {}
+        for name, value in data.items():
+            settings[name] = {'global': value}
+        return settings
 
-    def _handle_migrations(self):
-        """Handle unknown/renamed keys."""
-        for name in list(self._values):
+    def _build_values(self, settings):
+        """Build up self._values from the values in the given dict."""
+        errors = []
+        for name, yaml_values in settings.items():
+            if not isinstance(yaml_values, dict):
+                errors.append(configexc.ConfigErrorDesc(
+                    "While parsing {!r}".format(name), "value is not a dict"))
+                continue
+
+            values = configutils.Values(configdata.DATA[name])
+            if 'global' in yaml_values:
+                values.add(yaml_values.pop('global'))
+
+            for pattern, value in yaml_values.items():
+                if not isinstance(pattern, str):
+                    errors.append(configexc.ConfigErrorDesc(
+                        "While parsing {!r}".format(name),
+                        "pattern is not of type string"))
+                    continue
+                try:
+                    urlpattern = urlmatch.UrlPattern(pattern)
+                except urlmatch.ParseError as e:
+                    errors.append(configexc.ConfigErrorDesc(
+                        "While parsing pattern {!r} for {!r}"
+                        .format(pattern, name), e))
+                    continue
+                values.add(value, urlpattern)
+
+            self._values[name] = values
+
+        if errors:
+            raise configexc.ConfigFileErrors('autoconfig.yml', errors)
+
+    def _handle_migrations(self, settings):
+        """Migrate older configs to the newest format."""
+        # Simple renamed/deleted options
+        for name in list(settings):
             if name in configdata.MIGRATIONS.renamed:
                 new_name = configdata.MIGRATIONS.renamed[name]
                 log.config.debug("Renaming {} to {}".format(name, new_name))
-                self._values[new_name] = self._values[name]
-                del self._values[name]
+                settings[new_name] = settings[name]
+                del settings[name]
+                self._mark_changed()
             elif name in configdata.MIGRATIONS.deleted:
                 log.config.debug("Removing {}".format(name))
-                del self._values[name]
-            elif name in configdata.DATA:
-                pass
-            else:
-                desc = configexc.ConfigErrorDesc(
-                    "While loading options",
-                    "Unknown option {}".format(name))
-                raise configexc.ConfigFileErrors('autoconfig.yml', [desc])
+                del settings[name]
+                self._mark_changed()
 
-    def unset(self, name):
-        """Remove the given option name if it's configured."""
-        try:
-            del self._values[name]
-        except KeyError:
-            return
+        # tabs.persist_mode_on_change got merged into tabs.mode_on_change
+        old = 'tabs.persist_mode_on_change'
+        new = 'tabs.mode_on_change'
+        if old in settings:
+            settings[new] = {}
+            for scope, val in settings[old].items():
+                if val:
+                    settings[new][scope] = 'persist'
+                else:
+                    settings[new][scope] = 'normal'
+
+            del settings[old]
+            self._mark_changed()
+
+        # bindings.default can't be set in autoconfig.yml anymore, so ignore
+        # old values.
+        if 'bindings.default' in settings:
+            del settings['bindings.default']
+            self._mark_changed()
+
+        return settings
+
+    def _validate(self, settings):
+        """Make sure all settings exist."""
+        unknown = []
+        for name in settings:
+            if name not in configdata.DATA:
+                unknown.append(name)
+
+        if unknown:
+            errors = [configexc.ConfigErrorDesc("While loading options",
+                                                "Unknown option {}".format(e))
+                      for e in sorted(unknown)]
+            raise configexc.ConfigFileErrors('autoconfig.yml', errors)
+
+    def set_obj(self, name, value, *, pattern=None):
+        """Set the given setting to the given value."""
+        self._values[name].add(value, pattern)
         self._mark_changed()
 
+    def unset(self, name, *, pattern=None):
+        """Remove the given option name if it's configured."""
+        changed = self._values[name].remove(pattern)
+        if changed:
+            self._mark_changed()
+
     def clear(self):
         """Clear all values from the YAML file."""
-        self._values = []
+        for values in self._values.values():
+            values.clear()
         self._mark_changed()
 
 
@@ -225,6 +326,7 @@ class ConfigAPI:
 
     @contextlib.contextmanager
     def _handle_error(self, action, name):
+        """Catch config-related exceptions and save them in self.errors."""
         try:
             yield
         except configexc.ConfigFileErrors as e:
@@ -234,30 +336,65 @@ class ConfigAPI:
         except configexc.Error as e:
             text = "While {} '{}'".format(action, name)
             self.errors.append(configexc.ConfigErrorDesc(text, e))
+        except urlmatch.ParseError as e:
+            text = "While {} '{}' and parsing pattern".format(action, name)
+            self.errors.append(configexc.ConfigErrorDesc(text, e))
+        except keyutils.KeyParseError as e:
+            text = "While {} '{}' and parsing key".format(action, name)
+            self.errors.append(configexc.ConfigErrorDesc(text, e))
 
     def finalize(self):
         """Do work which needs to be done after reading config.py."""
         self._config.update_mutables()
 
     def load_autoconfig(self):
+        """Load the autoconfig.yml file which is used for :set/:bind/etc."""
         with self._handle_error('reading', 'autoconfig.yml'):
             read_autoconfig()
 
-    def get(self, name):
+    def get(self, name, pattern=None):
+        """Get a setting value from the config, optionally with a pattern."""
         with self._handle_error('getting', name):
-            return self._config.get_obj(name)
+            urlpattern = urlmatch.UrlPattern(pattern) if pattern else None
+            return self._config.get_mutable_obj(name, pattern=urlpattern)
 
-    def set(self, name, value):
+    def set(self, name, value, pattern=None):
+        """Set a setting value in the config, optionally with a pattern."""
         with self._handle_error('setting', name):
-            self._config.set_obj(name, value)
+            urlpattern = urlmatch.UrlPattern(pattern) if pattern else None
+            self._config.set_obj(name, value, pattern=urlpattern)
 
     def bind(self, key, command, mode='normal'):
+        """Bind a key to a command, with an optional key mode."""
         with self._handle_error('binding', key):
-            self._keyconfig.bind(key, command, mode=mode)
+            seq = keyutils.KeySequence.parse(key)
+            self._keyconfig.bind(seq, command, mode=mode)
 
     def unbind(self, key, mode='normal'):
+        """Unbind a key from a command, with an optional key mode."""
         with self._handle_error('unbinding', key):
-            self._keyconfig.unbind(key, mode=mode)
+            seq = keyutils.KeySequence.parse(key)
+            self._keyconfig.unbind(seq, mode=mode)
+
+    def source(self, filename):
+        """Read the given config file from disk."""
+        if not os.path.isabs(filename):
+            filename = str(self.configdir / filename)
+
+        try:
+            read_config_py(filename)
+        except configexc.ConfigFileErrors as e:
+            self.errors += e.errors
+
+    @contextlib.contextmanager
+    def pattern(self, pattern):
+        """Get a ConfigContainer for the given pattern."""
+        # We need to propagate the exception so we don't need to return
+        # something.
+        urlpattern = urlmatch.UrlPattern(pattern)
+        container = config.ConfigContainer(config=self._config, configapi=self,
+                                           pattern=urlpattern)
+        yield container
 
 
 class ConfigPyWriter:
@@ -317,7 +454,7 @@ class ConfigPyWriter:
 
     def _gen_options(self):
         """Generate the options part of the config."""
-        for opt, value in self._options:
+        for pattern, opt, value in self._options:
             if opt.name in ['bindings.commands', 'bindings.default']:
                 continue
 
@@ -336,7 +473,11 @@ class ConfigPyWriter:
                     except KeyError:
                         yield self._line("#   - {}".format(val))
 
-            yield self._line('c.{} = {!r}'.format(opt.name, value))
+            if pattern is None:
+                yield self._line('c.{} = {!r}'.format(opt.name, value))
+            else:
+                yield self._line('config.set({!r}, {!r}, {!r})'.format(
+                    opt.name, value, str(pattern)))
             yield ''
 
     def _gen_bindings(self):
@@ -344,15 +485,17 @@ class ConfigPyWriter:
         normal_bindings = self._bindings.pop('normal', {})
         if normal_bindings:
             yield self._line('# Bindings for normal mode')
-        for key, command in sorted(normal_bindings.items()):
-            yield self._line('config.bind({!r}, {!r})'.format(key, command))
+            for key, command in sorted(normal_bindings.items()):
+                yield self._line('config.bind({!r}, {!r})'.format(
+                    key, command))
+            yield ''
 
         for mode, mode_bindings in sorted(self._bindings.items()):
-            yield ''
             yield self._line('# Bindings for {} mode'.format(mode))
             for key, command in sorted(mode_bindings.items()):
                 yield self._line('config.bind({!r}, {!r}, mode={!r})'.format(
                     key, command, mode))
+            yield ''
 
 
 def read_config_py(filename, raising=False):
@@ -390,7 +533,7 @@ def read_config_py(filename, raising=False):
         desc = configexc.ConfigErrorDesc("Error while compiling", e)
         raise configexc.ConfigFileErrors(basename, [desc])
     except SyntaxError as e:
-        desc = configexc.ConfigErrorDesc("Syntax Error", e,
+        desc = configexc.ConfigErrorDesc("Unhandled exception", e,
                                          traceback=traceback.format_exc())
         raise configexc.ConfigFileErrors(basename, [desc])
 
