@@ -114,7 +114,7 @@ class BaseKeyParser(QObject):
 
         return (result, None)
 
-    def handle(self, e, *, dry_run=False):
+    def handle(self, e, *, dry_run=False):  # noqa
         """Handle a new keypress.
 
         Separate the keypress into count/command, then check if it matches
@@ -147,10 +147,18 @@ class BaseKeyParser(QObject):
             return QKeySequence.NoMatch
 
         # First, try a straightforward match
+        self._debug_log("Trying simple match")
         match, binding = self._match_key(sequence)
 
+        # Then try without optional modifiers
+        if match == QKeySequence.NoMatch:
+            self._debug_log("Trying match without modifiers")
+            sequence = sequence.strip_modifiers()
+            match, binding = self._match_key(sequence)
+
         # If that doesn't match, try a key_mapping
         if match == QKeySequence.NoMatch:
+            self._debug_log("Trying match with key_mappings")
             mapped = sequence.with_mappings(config.val.bindings.key_mappings)
             if sequence != mapped:
                 self._debug_log("Mapped {} -> {}".format(
@@ -159,10 +167,12 @@ class BaseKeyParser(QObject):
                 sequence = mapped
 
         # If that doesn't match either, try treating it as count.
+        txt = str(sequence[-1])  # To account for sequences changed above.
         if (match == QKeySequence.NoMatch and
                 txt.isdigit() and
                 self._supports_count and
                 not (not self._count and txt == '0')):
+            self._debug_log("Trying match as count")
             assert len(txt) == 1, txt
             if not dry_run:
                 self._count += txt
