@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -171,7 +171,7 @@ class PromptQueue(QObject):
             # just queue it up for later.
             log.prompt.debug("Adding {} to queue.".format(question))
             self._queue.append(question)
-            return
+            return None
 
         if blocking:
             # If we're blocking we save the old question on the stack, so we
@@ -207,6 +207,7 @@ class PromptQueue(QObject):
             return question.answer
         else:
             question.completed.connect(self._pop_later)
+            return None
 
     @pyqtSlot(usertypes.KeyMode)
     def _on_mode_left(self, mode):
@@ -364,7 +365,7 @@ class PromptContainer(QWidget):
             widget.hide()
             widget.deleteLater()
 
-    @cmdutils.register(instance='prompt-container', hide=True, scope='window',
+    @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt,
                               usertypes.KeyMode.yesno])
     def prompt_accept(self, value=None):
@@ -388,21 +389,7 @@ class PromptContainer(QWidget):
             message.global_bridge.prompt_done.emit(self._prompt.KEY_MODE)
             question.done()
 
-    @cmdutils.register(instance='prompt-container', hide=True, scope='window',
-                       modes=[usertypes.KeyMode.yesno],
-                       deprecated='Use :prompt-accept yes instead!')
-    def prompt_yes(self):
-        """Answer yes to a yes/no prompt."""
-        self.prompt_accept('yes')
-
-    @cmdutils.register(instance='prompt-container', hide=True, scope='window',
-                       modes=[usertypes.KeyMode.yesno],
-                       deprecated='Use :prompt-accept no instead!')
-    def prompt_no(self):
-        """Answer no to a yes/no prompt."""
-        self.prompt_accept('no')
-
-    @cmdutils.register(instance='prompt-container', hide=True, scope='window',
+    @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt], maxsplit=0)
     def prompt_open_download(self, cmdline: str = None):
         """Immediately open a download.
@@ -421,7 +408,7 @@ class PromptContainer(QWidget):
         except UnsupportedOperationError:
             pass
 
-    @cmdutils.register(instance='prompt-container', hide=True, scope='window',
+    @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt])
     @cmdutils.argument('which', choices=['next', 'prev'])
     def prompt_item_focus(self, which):
@@ -435,6 +422,27 @@ class PromptContainer(QWidget):
         except UnsupportedOperationError:
             pass
 
+    @cmdutils.register(
+        instance='prompt-container', scope='window',
+        modes=[usertypes.KeyMode.prompt, usertypes.KeyMode.yesno])
+    def prompt_yank(self, sel=False):
+        """Yank URL to clipboard or primary selection.
+
+        Args:
+            sel: Use the primary selection instead of the clipboard.
+        """
+        question = self._prompt.question
+        if question.url is None:
+            message.error('No URL found.')
+            return
+        if sel and utils.supports_selection():
+            target = 'primary selection'
+        else:
+            sel = False
+            target = 'clipboard'
+        utils.set_clipboard(question.url, sel)
+        message.info("Yanked to {}: {}".format(target, question.url))
+
 
 class LineEdit(QLineEdit):
 
@@ -499,8 +507,8 @@ class _BasePrompt(QWidget):
         self._key_grid = QGridLayout()
         self._key_grid.setVerticalSpacing(0)
 
-        # The bindings are all in the 'prompt' mode, even for yesno prompts
-        all_bindings = config.key_instance.get_reverse_bindings_for('prompt')
+        all_bindings = config.key_instance.get_reverse_bindings_for(
+            self.KEY_MODE.name)
         labels = []
 
         for cmd, text in self._allowed_commands():
@@ -734,6 +742,7 @@ class DownloadFilenamePrompt(FilenamePrompt):
             ('prompt-accept', 'Accept'),
             ('leave-mode', 'Abort'),
             ('prompt-open-download', "Open download"),
+            ('prompt-yank', "Yank URL"),
         ]
         return cmds
 
@@ -824,6 +833,7 @@ class YesNoPrompt(_BasePrompt):
         cmds = [
             ('prompt-accept yes', "Yes"),
             ('prompt-accept no', "No"),
+            ('prompt-yank', "Yank URL"),
         ]
 
         if self.question.default is not None:
