@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,9 +24,10 @@ from PyQt5.QtWidgets import QSizePolicy
 
 from qutebrowser.keyinput import modeman, modeparsers
 from qutebrowser.commands import cmdexc, cmdutils
-from qutebrowser.misc import cmdhistory
+from qutebrowser.misc import cmdhistory, editor
 from qutebrowser.misc import miscwidgets as misc
-from qutebrowser.utils import usertypes, log, objreg
+from qutebrowser.utils import usertypes, log, objreg, message, utils
+from qutebrowser.config import config
 
 
 class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
@@ -66,6 +67,7 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         self.cursorPositionChanged.connect(self.update_completion)
         self.textChanged.connect(self.update_completion)
         self.textChanged.connect(self.updateGeometry)
+        self.textChanged.connect(self._incremental_search)
 
     def prefix(self):
         """Get the currently entered command prefix."""
@@ -124,7 +126,7 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         else:
             self.set_cmd_text(text)
 
-    @cmdutils.register(instance='status-command', hide=True,
+    @cmdutils.register(instance='status-command',
                        modes=[usertypes.KeyMode.command], scope='window')
     def command_history_prev(self):
         """Go back in the commandline history."""
@@ -139,7 +141,7 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         if item:
             self.set_cmd_text(item)
 
-    @cmdutils.register(instance='status-command', hide=True,
+    @cmdutils.register(instance='status-command',
                        modes=[usertypes.KeyMode.command], scope='window')
     def command_history_next(self):
         """Go forward in the commandline history."""
@@ -152,10 +154,14 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         if item:
             self.set_cmd_text(item)
 
-    @cmdutils.register(instance='status-command', hide=True,
+    @cmdutils.register(instance='status-command',
                        modes=[usertypes.KeyMode.command], scope='window')
-    def command_accept(self):
-        """Execute the command currently in the commandline."""
+    def command_accept(self, rapid=False):
+        """Execute the command currently in the commandline.
+
+        Args:
+            rapid: Run the command without closing or clearing the command bar.
+        """
         prefixes = {
             ':': '',
             '/': 'search -- ',
@@ -163,9 +169,33 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         }
         text = self.text()
         self.history.append(text)
-        modeman.leave(self._win_id, usertypes.KeyMode.command, 'cmd accept')
+        if not rapid:
+            modeman.leave(self._win_id, usertypes.KeyMode.command,
+                          'cmd accept')
         self.got_cmd[str].emit(prefixes[text[0]] + text[1:])
 
+    @cmdutils.register(instance='status-command', scope='window')
+    def edit_command(self, run=False):
+        """Open an editor to modify the current command.
+
+        Args:
+            run: Run the command if the editor exits successfully.
+        """
+        ed = editor.ExternalEditor(parent=self)
+
+        def callback(text):
+            """Set the commandline to the edited text."""
+            if not text or text[0] not in modeparsers.STARTCHARS:
+                message.error('command must start with one of {}'
+                              .format(modeparsers.STARTCHARS))
+                return
+            self.set_cmd_text(text)
+            if run:
+                self.command_accept()
+
+        ed.file_updated.connect(callback)
+        ed.edit(self.text())
+
     @pyqtSlot(usertypes.KeyMode)
     def on_mode_left(self, mode):
         """Clear up when command mode was left.
@@ -191,8 +221,8 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         elif text[0] in modeparsers.STARTCHARS:
             super().set_prompt(text[0])
         else:
-            raise AssertionError("setText got called with invalid text "
-                                 "'{}'!".format(text))
+            raise utils.Unreachable("setText got called with invalid text "
+                                    "'{}'!".format(text))
         super().setText(text)
 
     def keyPressEvent(self, e):
@@ -202,6 +232,12 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
         Enter/Shift+Enter/etc. will cause QLineEdit to think it's finished
         without command_accept to be called.
         """
+        text = self.text()
+        if text in modeparsers.STARTCHARS and e.key() == Qt.Key_Backspace:
+            e.accept()
+            modeman.leave(self._win_id, usertypes.KeyMode.command,
+                          'prefix deleted')
+            return
         if e.key() == Qt.Key_Return:
             e.ignore()
             return
@@ -216,3 +252,16 @@ class Command(misc.MinimalLineEditMixin, misc.CommandLineEdit):
             text = 'x'
         width = self.fontMetrics().width(text)
         return QSize(width, height)
+
+    @pyqtSlot(str)
+    def _incremental_search(self, text):
+        if not config.val.search.incremental:
+            return
+
+        search_prefixes = {
+            '/': 'search -- ',
+            '?': 'search -r -- ',
+        }
+
+        if self.prefix() in ['/', '?']:
+            self.got_cmd[str].emit(search_prefixes[text[0]] + text[1:])
