@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -39,6 +39,7 @@ from qutebrowser.utils import log, objreg, usertypes, message, debug, utils
 from qutebrowser.commands import cmdutils, runners, cmdexc
 from qutebrowser.config import config, configdata
 from qutebrowser.misc import consolewidget
+from qutebrowser.utils.version import pastebin_version
 
 
 @cmdutils.register(maxsplit=1, no_cmd_split=True, no_replace_variables=True)
@@ -369,8 +370,15 @@ def nop():
 
 @cmdutils.register()
 @cmdutils.argument('win_id', win_id=True)
-def version(win_id):
-    """Show version information."""
+def version(win_id, paste=False):
+    """Show version information.
+
+    Args:
+        paste: Paste to pastebin.
+    """
     tabbed_browser = objreg.get('tabbed-browser', scope='window',
                                 window=win_id)
     tabbed_browser.openurl(QUrl('qute://version'), newtab=True)
+
+    if paste:
+        pastebin_version()
